@@ -9,14 +9,9 @@
 This module implements the concept of Dictionary -- a mapping between words and 
 their internal ids.
 
-The actual process of id translation proceeds in three steps:
- 1. get the input word (eg. 'Answering')
- 2. map word to its normalized form (eg. 'answer')
- 3. map the normalized form to integer id (eg. 42)
-
 Dictionaries can be created from a corpus and can later be pruned according to
-document frequency (removing (un)common words via the filterExtremes() method), 
-save/loaded from disk via save() and load() methods etc.
+document frequency (removing (un)common words via the :func:`Dictionary.filterExtremes` method), 
+save/loaded from disk via :func:`Dictionary.save` and :func:`Dictionary.load` methods etc.
 """
 
 
@@ -54,50 +49,43 @@ class Token(object):
 
 class Dictionary(utils.SaveLoad):
     """
-    Dictionary encapsulates mappings between words, their normalized forms and ids
-    of those normalized forms.
+    Dictionary encapsulates mappings between normalized words and their integer ids.
     
     The main function is `doc2bow`, which coverts a collection of words to its bow 
-    representation, optionally also updating the dictionary mappings with new 
+    representation, optionally also updating the dictionary mapping with new 
     words and their ids.
     """
     def __init__(self):
         self.id2token = {} # tokenId -> token
         self.token2id = {} # token -> tokenId
-        self.word2id = {} # surface form (word as appearing in text) -> tokenId
         self.docFreq = {} # tokenId -> in how many documents this token appeared
         self.numDocs = 0
     
 
     def __len__(self):
         """
-        Return the number of word->id mappings in the dictionary.
+        Return the number of token->id mappings in the dictionary.
         """
         assert len(self.token2id) == len(self.id2token)
         return len(self.token2id)
 
 
     def __str__(self):
-        return ("Dictionary(%i unique tokens covering %i surface forms)" %
-                (len(self), len(self.word2id)))
+        return ("Dictionary(%i unique tokens)" % len(self))
 
     @staticmethod
-    def fromDocuments(documents, normalizeWord):
+    def fromDocuments(documents):
         """
         Build dictionary from a collection of documents. Each document is a list 
-        of words (ie. tokenized strings).
+        of tokens (ie. **tokenized and normalized** utf-8 encoded strings).
         
-        The normalizeWord function is used to convert each word to its utf-8 encoded
-        canonical form (identity, lowercasing, stemming, ...); use whichever normalization
-        suits you.
-        
-        >>> print Dictionary.fromDocuments(["máma mele maso".split(), "ema má mama".split()], utils.deaccent)
-        Dictionary(5 unique tokens covering 6 surface forms)
+        >>> print Dictionary.fromDocuments(["máma mele maso".split(), "ema má mama".split()])
+        Dictionary(6 unique tokens)
         
         """
         result = Dictionary()
         for document in documents:
-            _ = result.doc2bow(document, normalizeWord, allowUpdate = True) # ignore the result, here we only care about updating token ids
+            _ = result.doc2bow(document, allowUpdate = True) # ignore the result, here we only care about updating token ids
         return result
 
 
@@ -109,40 +97,34 @@ class Dictionary(utils.SaveLoad):
         self.token2id[token.token] = token.intId
     
     
-    def doc2bow(self, document, normalizeWord, allowUpdate = False):
+    def doc2bow(self, document, allowUpdate = False):
         """
-        Convert `document` (a list of words) into bag-of-words format = list of 
-        (tokenId, tokenCount) 2-tuples.
+        Convert `document` (a list of words) into the bag-of-words format = list of 
+        `(tokenId, tokenCount)` 2-tuples. Each word is assumed to be a 
+        **tokenized and normalized** utf-8 encoded string.
         
-        `normalizeWord` must be a function that accepts one utf-8 encoded string
-        and returns another. Possible choices are identity, lowercasing etc.
-        
-        If `allowUpdate` is set, then also update dictionary in the process: create ids 
+        If `allowUpdate` is set, then also update of dictionary in the process: create ids 
         for new words. At the same time, update document frequencies -- for 
         each word appearing in this document, increase its self.docFreq by one.
+        
+        If `allowUpdate` is **not** set, this function is `const`, ie. read-only.
         """
-        # construct (word, frequency) mapping. in python3 this is done simply 
-        # using Counter(), but here i use itertools.groupby()
         result = {}
-        for word, group in itertools.groupby(sorted(document)):
+        
+        # construct (word, frequency) mapping. in python3 this is done simply 
+        # using Counter(), but here i use itertools.groupby() for the job
+        for wordNorm, group in itertools.groupby(sorted(document)):
             frequency = len(list(group)) # how many times does this word appear in the input document
 
             # determine the Token object of this word, creating it if necessary
-            tokenId = self.word2id.get(word, None)
-            if tokenId is None:
-                # first time we see this surface form
-                wordNorm = normalizeWord(word)
-                tokenId = self.token2id.get(wordNorm, None)
-                if tokenId is None: 
-                    # first time we see this token (normalized form)
-                    if not allowUpdate: # if we aren't allowed to create new tokens, continue with the next word
-                        continue
-                    tokenId = len(self.token2id)
-                    token = Token(wordNorm, tokenId)
-                    self.addToken(token)
-                else:
-                    token = self.id2token[tokenId]
-                self.word2id[word] = tokenId
+            tokenId = self.token2id.get(wordNorm, None)
+            if tokenId is None: 
+                # first time we see this token (normalized form)
+                if not allowUpdate: # if we aren't allowed to create new tokens, continue with the next unique token
+                    continue
+                tokenId = len(self.token2id) # new id = number of ids made so far; NOTE this assumes there are no gaps in the id sequence!
+                token = Token(wordNorm, tokenId)
+                self.addToken(token)
             else:
                 token = self.id2token[tokenId]
             # post condition -- now both tokenId and token object are properly set
@@ -156,20 +138,20 @@ class Dictionary(utils.SaveLoad):
             for tokenId in result.iterkeys():
                 self.docFreq[tokenId] = self.docFreq.get(tokenId, 0) + 1
         
-        return sorted(result.iteritems()) # return tokenIds in ascending order
+        return sorted(result.iteritems()) # return tokenIds, in ascending id order
 
 
     def filterExtremes(self, noBelow = 5, noAbove = 0.5):
         """
-        Filter out tokens that appear in 
-         1. less than `noBelow` documents (absolute number) or 
-         2. more than `noAbove` documents (fraction of total corpus size, *not* 
-            absolute number).
+        Filter out tokens that appear in
         
-        At the same time rebuild the dictionary, shrinking resulting gaps in 
-        tokenIds (lowering len(self) and freeing up memory in the process). 
+        1. less than `noBelow` documents (absolute number) or 
+        2. more than `noAbove` documents (fraction of total corpus size, *not* 
+           absolute number).
         
-        Note that the same token may have a different `tokenId` before and after
+        After the pruning, shrink resulting gaps in word ids. 
+        
+        **Note**: The same word may have a different word id before and after
         the call to this function!
         """
         noAboveAbs = int(noAbove * self.numDocs) # convert fractional threshold to absolute threshold
@@ -195,6 +177,8 @@ class Dictionary(utils.SaveLoad):
     def filterTokens(self, badIds):
         """
         Remove the selected tokens from all dictionary mappings.
+        
+        `badIds` is a collection of word ids to be removed.
         """
         badIds = set(badIds)
         self.id2token = dict((tokenId, token) for tokenId, token in self.id2token.iteritems() if tokenId not in badIds)
@@ -205,10 +189,10 @@ class Dictionary(utils.SaveLoad):
     
     def rebuildDictionary(self):
         """
-        Assign new tokenIds to all tokens. 
+        Assign new word ids to all words. 
         
-        This is done to make tokenIds more compact, ie. after some tokens have 
-        been removed via filterTokens() and there are gaps in the tokenId series.
+        This is done to make the ids more compact, ie. after some tokens have 
+        been removed via :func:`filterTokens` and there are gaps in the id series.
         Calling this method will remove the gaps.
         """
         logging.debug("rebuilding dictionary, shrinking gaps")
